/**
 * FTColumnflow is a polyfill that fixes the inadequacies of CSS column layouts.
 *
 * It is developed by FT Labs (http://labs.ft.com), part of the Financial Times.
 * It is extensively used in the FT Web App (http://app.ft.com), where it allows us to
 * publish articles with complex newspaper/magazine style layouts, including features such as:
 *    - Configurable column widths, gutters and margins
 *    - Fixed-position elements
 *    - Elements spanning columns
 *    - Keep-with-next to avoid headings at the bottom of a column
 *    - No-wrap class to avoid splitting elements across columns
 *    - Grouping of columns into pages
 *    - Horizontal or vertical alignment of pages
 *    - Standardised line height to align text baseline to a grid
 *    - Rapid reflow as required by events such as device orientation or font-size change
 * It is designed with the same column dimension specification API as the CSS3 multi-column specification
 * (http://www.w3.org/TR/css3-multicol/), but gives far greater flexibility over element positioning within those columns.
 *
 * @codingstandard ftlabs-jslint
 * @copyright The Financial Times Limited [All Rights Reserved]
*/
this.FTColumnflow=(function(){"use strict";function FTColumnflowException(name,message){this.name='FTColumnflow'+name||'FTColumnflowException';this.message=message||''}FTColumnflowException.prototype=new Error();FTColumnflowException.constructor=FTColumnflowException;var layoutDimensionList=['pageInnerWidth','pageInnerHeight','colDefaultTop','colDefaultLeft','columnCount','columnWidth','columnGap'],defaultConfig={layoutDimensions:null,columnFragmentMinHeight:0,viewportWidth:null,viewportHeight:null,columnWidth:'auto',columnCount:'auto',columnGap:'normal',pageClass:'cf-page',columnClass:'cf-column',pageArrangement:'horizontal',pagePadding:0,debug:false,showGrid:false,standardiseLineHeight:false,minFixedPadding:1,lineHeight:null,noWrapOnTags:[]},cssStyles='#[targetId] { position: relative; height: 100%; }\n'+'#[targetId] .[preloadAreaClassName].[pageClass] { visibility: hidden; position: absolute; overflow: hidden; }\n'+'#[targetId] .[preloadFixedAreaClassName] { visibility: hidden; position: absolute; }\n'+'#[targetId] .[pageClass] { position: absolute; width: [viewportWidth]px; height: [viewportHeight]px; [pageArrangement] }\n'+'#[targetId] .[columnClass] { position: absolute; width: [columnWidth]px; overflow: hidden; }\n'+'#[targetId] .[pageClass] .[fixedElementClassName] { position: absolute; }\n'+'#[targetId] .[pageClass] .[columnClass] > :first-child { margin-top: 0px; }\n',cssColumnStyles='#[targetId] .[columnClass].[columnClass]-[columnNum] { left: [leftPos]px; }\n',showGridStyles='#[targetId] .[pageClass] { background-image: -webkit-linear-gradient(skyblue 1px, transparent 1px); background-size: 100% [lh]px; background-origin: content-box; }',_outerHTML=(function(){var outerHTMLXmlSerializer,outerHTMLContainer,html;if(typeof document!=="undefined"&&!document.createElementNS("http://www.w3.org/1999/xhtml","_").hasOwnProperty('outerHTML')){if(document.xmlVersion){outerHTMLXmlSerializer=new XMLSerializer();return function _outerHTMLXML(node){return outerHTMLXmlSerializer.serializeToString(node)}}else{outerHTMLContainer=document.createElementNS("http://www.w3.org/1999/xhtml","_");return function _outerHTMLNode(node){var html;outerHTMLContainer.appendChild(node.cloneNode(false));html=outerHTMLContainer.innerHTML.replace("><",">"+node.innerHTML+"<");outerHTMLContainer.innerHTML="";return html}}}else{return function _outerHTMLNative(node){return node.outerHTML}}}());function FTColumnflow(target,viewport,userConfig){var config={},debugMode,showGrid,flowedContent,fixedContent,maxColumnHeight,colDefaultBottom,colMiddle,minFixedPadding,fixedPadding,renderArea,preloadColumn,fixedPreloadArea,headElement,headerStyles,targetIdPrefix='cf-target-',renderAreaClassName='cf-render-area',preloadAreaClassName='cf-preload',preloadFixedAreaClassName='cf-preload-fixed',fixedElementClassName='cf-fixed',nowrapClassName='nowrap',keepwithnextClassName='keepwithnext',lineHeightTestContents,pagedContent=[],borderElementIndex,indexedPageNum,indexedColumnNum,indexedColumnFrag,topElementOverflow,totalColumnHeight,workingPage,workingColumn,workingColumnFrag,that=this;this.target=target;this.viewport=viewport;this._checkInstanceArgs();_setConfig(userConfig);_setLayoutDimensions();function _setConfig(userConfig){var i;that.config=config={};for(i in defaultConfig){if(defaultConfig.hasOwnProperty(i)){config[i]=defaultConfig[i]}}for(i in userConfig){if(userConfig.hasOwnProperty(i)){if(config[i]===undefined){throw new FTColumnflowException('ParameterException','Unknown config parameter ['+i+'].')}config[i]=userConfig[i]}}if(config.debug!==undefined)debugMode=!!config.debug;if(config.showGrid!==undefined)showGrid=!!config.showGrid;if('horizontal'!==config.pageArrangement&&'vertical'!==config.pageArrangement){throw new FTColumnflowException('ArrangementException',config.pageArrangement+' is not a valid Page Arrangement value.')}config.pagePadding=parseInt(config.pagePadding,10);if(isNaN(config.pagePadding)){throw new FTColumnflowException('PaddingException',config.pagePadding+' is not a valid Page Padding value.')}['columnWidth','columnCount','columnGap','columnFragmentMinHeight'].forEach(function _checkColumnSpecification(type){if(defaultConfig[type]===config[type])return true;config[type]=parseInt(config[type],10);if(isNaN(config[type])||config[type]<0){throw new FTColumnflowException('ColumnDimensionException',type+' must be an positive integer or "'+defaultConfig[type]+'".')}});if(typeof config.standardiseLineHeight!=='boolean'){throw new FTColumnflowException('StandardiseLineheightException','standardiseLineHeight must be a boolean value.')}config.minFixedPadding=parseFloat(config.minFixedPadding);if(isNaN(config.minFixedPadding)){throw new FTColumnflowException('MinFixedPaddingException','minFixedPadding must be a float or integer.')}if((config.viewportWidth!==null)&&isNaN(config.viewportWidth=parseInt(config.viewportWidth,10))){throw new FTColumnflowException('ViewportWidthException','viewportWidth must be an integer.')}if((config.viewportHeight!==null)&&isNaN(config.viewportHeight=parseInt(config.viewportHeight,10))){throw new FTColumnflowException('ViewportHeightException','viewportHeight must be an integer.')}if((config.lineHeight!==null)&&isNaN(config.lineHeight=parseInt(config.lineHeight,10))){throw new FTColumnflowException('LineheightException','lineHeight must be an integer.')}config.pageClass=_normaliseClassName('pageClass',config.pageClass);config.columnClass=_normaliseClassName('columnClass',config.columnClass);if(!Array.isArray(config.noWrapOnTags)){throw new FTColumnflowException('NoWrapException','noWrapOnTags must be an Array.')}config.noWrapOnTags=config.noWrapOnTags.map(function _lowercase(item){return item.toLowerCase()})}function _setLayoutDimensions(){var i,l,derivedColumnCount;if(config.layoutDimensions!==null){for(i=0,l=layoutDimensionList.length;i<l;i++){if(isNaN(Number(config.layoutDimensions[layoutDimensionList[i]]))){throw new FTColumnflowException('DimensionCacheException','Must specify an integer value for '+layoutDimensionList[i])}}return}config.layoutDimensions={};if(!config.viewportWidth){config.viewportWidth=parseInt(window.getComputedStyle(that.viewport).getPropertyValue('width'),10)}if(!config.viewportHeight){config.viewportHeight=parseInt(window.getComputedStyle(that.viewport).getPropertyValue('height'),10)}if(!config.viewportWidth||!config.viewportHeight){throw new FTColumnflowException('ViewportException','Viewport element must have width and height.')}config.layoutDimensions.columnGap=('normal'===config.columnGap)?parseInt(window.getComputedStyle(that.viewport).fontSize,10):config.columnGap;if('horizontal'===config.pageArrangement){config.layoutDimensions.pageInnerWidth=config.viewportWidth-(2*config.pagePadding);config.layoutDimensions.pageInnerHeight=config.viewportHeight;config.layoutDimensions.colDefaultTop=0;config.layoutDimensions.colDefaultLeft=config.pagePadding}else{config.layoutDimensions.pageInnerWidth=config.viewportWidth;config.layoutDimensions.pageInnerHeight=config.viewportHeight-(2*config.pagePadding);config.layoutDimensions.colDefaultTop=config.pagePadding;config.layoutDimensions.colDefaultLeft=0}if('auto'===config.columnWidth&&'auto'===config.columnCount){config.layoutDimensions.columnCount=1;config.layoutDimensions.columnWidth=config.layoutDimensions.pageInnerWidth}else if('auto'===config.columnWidth&&'auto'!==config.columnCount){config.layoutDimensions.columnCount=config.columnCount;config.layoutDimensions.columnWidth=(config.layoutDimensions.pageInnerWidth-((config.columnCount-1)*config.layoutDimensions.columnGap))/config.columnCount}else{derivedColumnCount=Math.max(1,Math.floor((config.layoutDimensions.pageInnerWidth+1+config.layoutDimensions.columnGap)/(config.columnWidth+config.layoutDimensions.columnGap)));if('auto'===config.columnCount){config.layoutDimensions.columnCount=derivedColumnCount;config.layoutDimensions.columnWidth=((config.layoutDimensions.pageInnerWidth+config.layoutDimensions.columnGap)/config.layoutDimensions.columnCount)-config.layoutDimensions.columnGap}else{config.layoutDimensions.columnCount=Math.min(config.columnCount,derivedColumnCount);config.layoutDimensions.columnWidth=((config.layoutDimensions.pageInnerWidth+config.layoutDimensions.columnGap)/config.layoutDimensions.columnCount)-config.layoutDimensions.columnGap}}}function _writeTargetStyles(){var styleContents,columnNum;if(!headElement)headElement=document.querySelector('head');if(!that.target.id)that.target.id=targetIdPrefix+Math.floor(Math.random()*(9000000000)+1000000000);styleContents=_replaceStringTokens(cssStyles,{targetId:that.target.id,preloadAreaClassName:preloadAreaClassName,preloadFixedAreaClassName:preloadFixedAreaClassName,fixedElementClassName:fixedElementClassName,pageClass:config.pageClass,columnClass:config.columnClass,columnWidth:config.layoutDimensions.columnWidth,viewportWidth:config.viewportWidth,viewportHeight:config.viewportHeight,pageArrangement:('horizontal'===config.pageArrangement)?'top: 0;':'left: 0;'});for(columnNum=1;columnNum<=config.layoutDimensions.columnCount;columnNum++){styleContents+=_replaceStringTokens(cssColumnStyles,{targetId:that.target.id,columnClass:config.columnClass,columnNum:columnNum,leftPos:config.layoutDimensions.colDefaultLeft+(config.layoutDimensions.columnWidth*(columnNum-1))+(config.layoutDimensions.columnGap*(columnNum-1))})}if(headerStyles){while(headerStyles.hasChildNodes()){headerStyles.removeChild(headerStyles.firstChild)}}else{headerStyles=document.createElement('style');headerStyles.setAttribute('type','text/css')}headerStyles.appendChild(document.createTextNode(styleContents));headElement.appendChild(headerStyles)}function _createTargetElements(){var preloadElement;that.target.innerHTML='<div class="'+preloadAreaClassName+' '+config.pageClass+'">'+'<div class="'+config.columnClass+'"></div>'+'</div>'+'<div class="'+preloadFixedAreaClassName+'"></div>'+'<div class="'+renderAreaClassName+'"></div>';renderArea=that.target.getElementsByClassName(renderAreaClassName)[0];preloadElement=that.target.getElementsByClassName(preloadAreaClassName)[0];preloadColumn=preloadElement.getElementsByClassName(config.columnClass)[0];if('string'===typeof flowedContent){preloadColumn.innerHTML=flowedContent}else if(flowedContent instanceof HTMLElement){preloadColumn.innerHTML=flowedContent.innerHTML}else if(flowedContent){throw new FTColumnflowException('FlowedContentException','flowedContent must be a HTML string or a DOM element.')}fixedPreloadArea=that.target.getElementsByClassName(preloadFixedAreaClassName)[0];if('string'===typeof fixedContent){fixedPreloadArea.innerHTML=fixedContent}else if(fixedContent instanceof HTMLElement){fixedPreloadArea.innerHTML=fixedContent.innerHTML}else if(fixedContent){throw new FTColumnflowException('FixedContentException','fixedContent must be a HTML string or a DOM element.')}}function _findLineHeight(){var lineHeights,i,l,node,testNode,testLineHeight;if(!config.lineHeight){if(!lineHeightTestContents){lineHeightTestContents=new Array(10).join('x<br />')+'x'}lineHeights=[];for(i=0,l=Math.min(10,(preloadColumn.childNodes.length));i<l;i++){node=preloadColumn.childNodes[i];if(Node.ELEMENT_NODE!==node.nodeType)continue;testLineHeight=parseInt(window.getComputedStyle(node).getPropertyValue('line-height'),10);if(!testLineHeight){testNode=node.cloneNode(false);if(node.className)testNode.className=node.className;testNode.style.padding="0px";testNode.style.border="none";testNode.style.height="auto";testNode.innerHTML=lineHeightTestContents;preloadColumn.appendChild(testNode);testLineHeight=testNode.offsetHeight/10;preloadColumn.removeChild(testNode)}lineHeights.push(testLineHeight)}if(lineHeights.length<5){testNode=document.createElement('p');testNode.style.padding="0px";testNode.style.border="none";testNode.style.height="auto";testNode.innerHTML=lineHeightTestContents;preloadColumn.appendChild(testNode);testLineHeight=testNode.offsetHeight/10;preloadColumn.removeChild(testNode);for(i=lineHeights.length;i<5;i++)lineHeights.push(testLineHeight)}config.lineHeight=_mode(lineHeights)}if(showGrid){headerStyles.innerHTML+=_replaceStringTokens(showGridStyles,{targetId:that.target.id,pageClass:config.pageClass,'lh':config.lineHeight})}}function _addFixedElement(element){var indexedColStart,indexedColEnd,anchorY,anchorX,elementTopPos,elementBottomPos,lowestTopPos,highestBottomPos,topSplitPoint,bottomSplitPoint,matches,indexedPageNum,spanDir,colSpan,normalisedElementHeight,columnNum,firstColFragment,lastColFragment,newColumnFragment,newFragmentHeight,fragment,column,fragNum,fragLen;if(Node.TEXT_NODE===element.nodeType||('none'===window.getComputedStyle(element).getPropertyValue('display'))){return}element.classList.add(fixedElementClassName);matches=element.className.match(/(\s|^)attach-page-(\d+)(\s|$)/);if(matches){indexedPageNum=matches[2]-1}else{indexedPageNum=0}_createPageObjects(indexedPageNum);workingPage=pagedContent[indexedPageNum];matches=element.className.match(/(\s|^)anchor-(top|middle|bottom)-(left|right|(?:col-(\d+)))(\s|$)/);if(matches){anchorY=matches[2];if(matches[4]){anchorX=Math.max(0,(Math.min(matches[4],config.layoutDimensions.columnCount)-1))}else{anchorX=matches[3]}}else{anchorY='top';anchorX='left'}matches=element.className.match(/(\s|^)col-span-(\d+|all)(-(left|right))?(\s|$)/);if(matches){spanDir=matches[4]||'right';if(matches[2]==='all'){colSpan=config.layoutDimensions.columnCount}else{colSpan=parseInt(matches[2],10)}if('left'===anchorX){indexedColStart=0;indexedColEnd=Math.min(colSpan,config.layoutDimensions.columnCount)-1}else if('right'===anchorX){indexedColEnd=config.layoutDimensions.columnCount-1;indexedColStart=config.layoutDimensions.columnCount-Math.min(colSpan,config.layoutDimensions.columnCount)}else{if('right'===spanDir){indexedColStart=anchorX;indexedColEnd=Math.min((indexedColStart+colSpan),config.layoutDimensions.columnCount)-1}else{indexedColEnd=anchorX;indexedColStart=Math.max((indexedColEnd-colSpan-1),0)}}}else{if('left'===anchorX){indexedColStart=0}else if('right'===anchorX){indexedColStart=config.layoutDimensions.columnCount-1}else{indexedColStart=anchorX}indexedColEnd=indexedColStart}element.style.width=((indexedColEnd-indexedColStart)*(config.layoutDimensions.columnWidth+config.layoutDimensions.columnGap))+config.layoutDimensions.columnWidth+'px';normalisedElementHeight=element.offsetHeight+parseInt(window.getComputedStyle(element).getPropertyValue('margin-top'),10);switch(anchorY){case'top':elementTopPos=config.layoutDimensions.colDefaultTop;lowestTopPos=colDefaultBottom-normalisedElementHeight;for(columnNum=indexedColStart;columnNum<=indexedColEnd;columnNum++){firstColFragment=workingPage.columns[columnNum].fragments[0];if(!firstColFragment){elementTopPos=lowestTopPos}else{if(firstColFragment.top>elementTopPos){elementTopPos=(firstColFragment.top>lowestTopPos)?lowestTopPos:firstColFragment.top}}}elementBottomPos=elementTopPos+normalisedElementHeight;topSplitPoint=elementTopPos-config.lineHeight;bottomSplitPoint=_roundUpToGrid(elementBottomPos,true);break;case'middle':elementTopPos=colMiddle-(normalisedElementHeight/2);topSplitPoint=_roundDownToGrid(elementTopPos,true);bottomSplitPoint=_roundUpToGrid(elementTopPos+normalisedElementHeight,true);if(topSplitPoint<0)topSplitPoint=0;if(bottomSplitPoint>maxColumnHeight)bottomSplitPoint=maxColumnHeight;break;case'bottom':elementBottomPos=colDefaultBottom;highestBottomPos=normalisedElementHeight;for(columnNum=indexedColStart;columnNum<=indexedColEnd;columnNum++){lastColFragment=workingPage.columns[columnNum].fragments[workingPage.columns[columnNum].fragments.length-1];if(!lastColFragment){elementBottomPos=highestBottomPos}else{if(lastColFragment.bottom<elementBottomPos){elementBottomPos=(lastColFragment.bottom<highestBottomPos)?highestBottomPos:lastColFragment.bottom}}}elementTopPos=elementBottomPos-normalisedElementHeight;topSplitPoint=_roundDownToGrid(elementTopPos,true);bottomSplitPoint=elementBottomPos+config.lineHeight;break}for(columnNum=indexedColStart;columnNum<=indexedColEnd;columnNum++){column=workingPage.columns[columnNum];for(fragNum=0,fragLen=column.fragments.length;fragNum<fragLen;fragNum++){fragment=column.fragments[fragNum];if(topSplitPoint<fragment.top&&bottomSplitPoint>fragment.bottom){column.fragments.splice(fragNum,1);fragLen--;continue}else if(topSplitPoint>fragment.bottom||bottomSplitPoint<fragment.top){continue}newFragmentHeight=fragment.top+fragment.height-bottomSplitPoint;fragment.height=topSplitPoint-fragment.top;fragment.bottom=fragment.top+fragment.height;if(!fragment.height||fragment.height<config.columnFragmentMinHeight){column.fragments.splice(fragNum--,1);fragLen--}if(newFragmentHeight&&newFragmentHeight>=config.columnFragmentMinHeight){newColumnFragment=_createColumnFragment();newColumnFragment.top=bottomSplitPoint;newColumnFragment.height=newFragmentHeight;newColumnFragment.bottom=newColumnFragment.top+newColumnFragment.height;column.fragments.splice(++fragNum,0,newColumnFragment);fragNum++;fragLen++}}}element.style.top=elementTopPos+'px';element.style.left=(config.layoutDimensions.colDefaultLeft+(('left'===anchorX)?0:((config.layoutDimensions.columnWidth+config.layoutDimensions.columnGap)*indexedColStart)))+'px';workingPage.fixed.push({content:_outerHTML(element)})}function _normaliseFlowedElement(element){var p;if(Node.TEXT_NODE!==element.nodeType)return;if(element.nodeValue.match(/^\s*$/)){element.parentNode.removeChild(element)}else{p=document.createElement('p');p.appendChild(document.createTextNode(element.nodeValue));element.parentNode.replaceChild(p,element)}}function _flowContent(){var i,l;pagedContent=[];indexedPageNum=indexedColumnNum=indexedColumnFrag=borderElementIndex=indexedPageNum=indexedColumnNum=indexedColumnFrag=topElementOverflow=totalColumnHeight=0;maxColumnHeight=config.lineHeight?_roundDownToGrid(config.layoutDimensions.pageInnerHeight):config.layoutDimensions.pageInnerHeight;colDefaultBottom=maxColumnHeight+config.layoutDimensions.colDefaultTop;colMiddle=config.layoutDimensions.colDefaultTop+(maxColumnHeight/2);minFixedPadding=config.minFixedPadding*config.lineHeight;fixedPadding=_roundUpToGrid(minFixedPadding);for(i=0,l=fixedPreloadArea.childNodes.length;i<l;i++){_addFixedElement(fixedPreloadArea.childNodes[i])}for(i=preloadColumn.childNodes.length-1;i>=0;i--){_normaliseFlowedElement(preloadColumn.childNodes[i])}if(!preloadColumn.childNodes.length)return;_createPageObjects(indexedPageNum);workingPage=pagedContent[indexedPageNum];workingColumn=workingPage.columns[indexedColumnNum];workingColumnFrag=workingColumn.fragments[indexedColumnFrag];if(!workingColumnFrag){_advanceWorkingColumnFragment()}totalColumnHeight=workingColumnFrag.height;for(i=0,l=preloadColumn.childNodes.length;i<l;i++){_addFlowedElement(preloadColumn.childNodes[i],i)}_wrapColumn(l-1,false)}function _addFlowedElement(element,index){var originalPadding,existingPadding,totalElementHeight,desiredElementHeight,newPadding,overflow,loopCount,nextElement=element.nextSibling;if(config.standardiseLineHeight){originalPadding=parseInt(element.getAttribute('data-cf-original-padding'),10)||null;existingPadding=parseInt(window.getComputedStyle(element).getPropertyValue('padding-bottom'),10);if(null===originalPadding){originalPadding=existingPadding;element.setAttribute('data-cf-original-padding',originalPadding)}else{existingPadding=originalPadding}if(originalPadding!==existingPadding){element.style.paddingBottom=originalPadding+'px'}totalElementHeight=nextElement?(nextElement.offsetTop-element.offsetTop):element.offsetHeight;desiredElementHeight=_roundUpToGrid(totalElementHeight);newPadding=desiredElementHeight-totalElementHeight+existingPadding;if(newPadding!==existingPadding){element.style.paddingBottom=newPadding+'px'}}element.offsetBottom=element.offsetTop+element.offsetHeight;loopCount=0;while((element.offsetBottom>=totalColumnHeight||(nextElement&&nextElement.offsetTop>=totalColumnHeight))&&(loopCount++<30)){overflow=(element.offsetBottom>totalColumnHeight);_wrapColumn(index,overflow)}if(loopCount>=30)console.error('FTColumnflow: Caught and destroyed a loop when wrapping columns for element',element.outerHTML.substr(0,200)+'...')}function _wrapColumn(currentElementIndex,overflow){var nowrap,keepwithnext,prevElementKeepwithnext,firstInColumn,finalColumnElementIndex,cropCurrentElement,pushElement,pushFromElementIndex,i,lastElement,element,currentElement=preloadColumn.childNodes[currentElementIndex],nextElement=currentElement.nextSibling,prevElement=currentElement.previousSibling;nowrap=currentElement.classList.contains(nowrapClassName);keepwithnext=currentElement.classList.contains(keepwithnextClassName);prevElementKeepwithnext=(prevElement&&prevElement.classList.contains(keepwithnextClassName));if(-1!==config.noWrapOnTags.indexOf(currentElement.tagName.toLowerCase())){nowrap=true}if(!nextElement){lastElement=true}if(currentElementIndex===borderElementIndex){firstInColumn=true}if(currentElement.offsetBottom===totalColumnHeight){overflow=false;if(nextElement)totalColumnHeight=nextElement.offsetTop}if((nowrap||(keepwithnext&&nextElement))&&overflow&&(firstInColumn)){cropCurrentElement=true}if(!cropCurrentElement&&!firstInColumn&&((nowrap&&overflow)||(keepwithnext&&nextElement))){pushElement=true;pushFromElementIndex=currentElementIndex}if((currentElementIndex-1)>borderElementIndex&&prevElementKeepwithnext&&overflow&&nowrap){pushElement=true;pushFromElementIndex=currentElementIndex-1}finalColumnElementIndex=pushElement?pushFromElementIndex-1:currentElementIndex;if(finalColumnElementIndex<borderElementIndex){return}workingColumnFrag.overflow=topElementOverflow;for(i=borderElementIndex;i<=finalColumnElementIndex;i++){element=preloadColumn.childNodes[i];workingColumnFrag.elements.push({content:_outerHTML(element)})}if(pushElement){borderElementIndex=pushFromElementIndex}else if(overflow&&!cropCurrentElement){borderElementIndex=currentElementIndex}else{borderElementIndex=currentElementIndex+1}if(pushElement){totalColumnHeight=preloadColumn.childNodes[pushFromElementIndex].offsetTop}else if(cropCurrentElement&&nextElement){totalColumnHeight=nextElement.offsetTop}if(!overflow||(nowrap||(keepwithnext&&nextElement))){topElementOverflow=0}else{topElementOverflow=totalColumnHeight-currentElement.offsetTop}_advanceWorkingColumnFragment();totalColumnHeight+=workingColumnFrag.height}function _renderFlowedContent(){var outputHTML='',indexedPageNum,page_len,pageHTML,page,i,l,element,indexedColumnNum,column_len,column,indexedColumnFrag,fragLen,el,fragment;for(indexedPageNum=0,page_len=pagedContent.length;indexedPageNum<page_len;indexedPageNum++){pageHTML='';page=pagedContent[indexedPageNum];for(i=0,l=page.fixed.length;i<l;i++){element=page.fixed[i];pageHTML+=element.content}for(indexedColumnNum=0,column_len=page.columns.length;indexedColumnNum<column_len;indexedColumnNum++){column=page.columns[indexedColumnNum];for(indexedColumnFrag=0,fragLen=column.fragments.length;indexedColumnFrag<fragLen;indexedColumnFrag++){fragment=column.fragments[indexedColumnFrag];if(0===fragment.elements.length){continue}pageHTML+=_openColumn(fragment,indexedColumnNum);for(el=0,l=fragment.elements.length;el<l;el++){element=fragment.elements[el];if(el===0){element.content=_addTopMargin(element.content,-fragment.overflow)}pageHTML+=element.content}pageHTML+='</div>'}}if(''===pageHTML){pagedContent.splice(indexedPageNum,1);indexedPageNum--;page_len--;continue}outputHTML+=_openPage(indexedPageNum)+pageHTML+'</div>'}renderArea.innerHTML=outputHTML;that.target.style.width=(config.viewportWidth*page_len)+'px';that.pagedContentCount=pagedContent.length}function _addTopMargin(element,margin){return element.replace(/<(\w+)([^>]*)>/,function _addTopMarginToTag(string,tag,attributes){if(!string.match(/style\s*=/)){string='<'+tag+' style="" '+attributes+'>'}string=string.replace(/style=(["'])/,'style=$1 margin-top:'+margin+'px;');return string})}function _roundDownToGrid(val,addPadding){var resized=val-(val%config.lineHeight);if(addPadding&&((val-resized)<minFixedPadding)){resized-=fixedPadding}return resized}function _roundUpToGrid(val,addPadding){var delta=val%config.lineHeight,resized=(delta?(val-delta+config.lineHeight):val);if(addPadding&&((resized-val)<minFixedPadding)){resized+=fixedPadding}return resized}function _replaceStringTokens(string,tokens){return string.replace(/\[(\w+)\]/g,function _replace(a,b){var r=tokens[b];return typeof r==='string'||typeof r==='number'?r:a})}function _normaliseClassName(type,value){if(typeof value!=='string'){throw new FTColumnflowException('ClassnameException',type+' must be a string.')}return value.replace(/[^\w]/g,'-')}function _createPageObjects(indexedPageNum){var pageNum,indexedColNum;for(pageNum=pagedContent.length;pageNum<=indexedPageNum;pageNum++){pagedContent.push({'fixed':[],'columns':[]});for(indexedColNum=0;indexedColNum<config.layoutDimensions.columnCount;indexedColNum++){pagedContent[pageNum].columns.push({fragments:[_createColumnFragment()]})}}}function _createColumnFragment(){return{elements:[],overflow:0,height:maxColumnHeight,top:config.layoutDimensions.colDefaultTop,bottom:colDefaultBottom}}function _advanceWorkingColumnFragment(){if(!workingColumn.fragments[++indexedColumnFrag]){indexedColumnFrag=0;if(!workingPage.columns[++indexedColumnNum]){indexedColumnNum=0;indexedPageNum++;_createPageObjects(indexedPageNum)}}workingPage=pagedContent[indexedPageNum];workingColumn=workingPage.columns[indexedColumnNum];workingColumnFrag=workingColumn.fragments[indexedColumnFrag];if(!workingColumnFrag){_advanceWorkingColumnFragment()}}function _openPage(indexedPageNum){var pagePos;if('horizontal'===config.pageArrangement){pagePos='left: '+(indexedPageNum*config.viewportWidth)+'px;'}else{pagePos='top: '+(indexedPageNum*config.viewportHeight)+'px;'}return'<div class="'+config.pageClass+' '+config.pageClass+'-'+(indexedPageNum+1)+'" style="'+pagePos+'">'}function _openColumn(column,indexedColumnNum){return'<div class="'+config.columnClass+' '+config.columnClass+'-'+(indexedColumnNum+1)+'" style="height: '+column.height+'px; top: '+column.top+'px;">'}function _mode(array){var modeMap={},maxEl,maxCount,i,el;if(array.length===0){return null}maxEl=array[0];maxCount=1;for(i=0;i<array.length;i++){el=array[i];if(modeMap[el]===undefined){modeMap[el]=1}else{modeMap[el]++}if(modeMap[el]>maxCount){maxEl=el;maxCount=modeMap[el]}}return maxEl}function _log(){if(!debugMode)return;console.log.apply(console,arguments)}this.flow=function(flowed,fixed){flowedContent=flowed;fixedContent=fixed;_writeTargetStyles();_createTargetElements();_findLineHeight();_flowContent();_renderFlowedContent()};this.reflow=function(newConfig){if(newConfig){_setConfig(newConfig)}_setLayoutDimensions();_writeTargetStyles();_findLineHeight();_flowContent();_renderFlowedContent()};this.destroy=function(){if(headerStyles){headerStyles.parentNode.removeChild(headerStyles);headerStyles=null}if(that.target){that.target.parentNode.removeChild(that.target);that.target=null}}}FTColumnflow.prototype={get layoutDimensions(){return this.config.layoutDimensions},set layoutDimensions(value){throw new FTColumnflowException('SetterException','Setter not defined for layoutDimensions.')},get pageClass(){return this.config.pageClass},set pageClass(value){throw new FTColumnflowException('SetterException','Setter not defined for pageClass.')},get columnClass(){return this.config.columnClass},set columnClass(value){throw new FTColumnflowException('SetterException','Setter not defined for columnClass.')},get pageCount(){return this.pagedContentCount},set pageCount(value){throw new FTColumnflowException('SetterException','Setter not defined for pageCount.')},_checkInstanceArgs:function(){var that=this;['target','viewport'].forEach(function _checkArg(name){var arg=that[name];switch(typeof arg){case'string':arg=document.getElementById(arg);if(!arg)throw new FTColumnflowException('SelectorException',name+' must be a valid DOM element.');break;case'object':if(!(arg instanceof HTMLElement)){throw new FTColumnflowException('ParameterException',name+' must be a string ID or DOM element.')}break;default:throw new FTColumnflowException('ParameterException',name+' must be a string ID or DOM element.')}that[name]=arg});if(!this._isDescendant(this.viewport,this.target)){throw new FTColumnflowException('InheritanceException','Target element must be a child of the viewport.')}this.target.innerHTML=''},_isDescendant:function(parent,child){var node=child.parentNode;while(node!==null){if(node===parent){return true}node=node.parentNode}return false}};return FTColumnflow}());